from langsearch.pipelines.detect import DetectItemTypePipeline
from langsearch.pipelines.dryrun import DryRunPipeline
from langsearch.pipelines.types.assemble import assemble
from langsearch.pipelines.types.html.htmlpipeline import GenericHTMLPipeline
